#!/usr/local/bin/python
#
# register.py - CGI program to register a verlof entry.
#
# Verlof Web application
#
# Copyright (c) 2002, by Leiden University.
# verlof is free software.  See the file Copying for copying permission.
#
# $Revision: 1.8 $
# $Date: 2003-01-27 14:06:10+01 $
# $Source: \\\\fwneld\\msdos\\RCS\\K\\fsc\\prj\\verlof\\src\\register.py,v $
# %Author: M.J. Moene %

#
# $Header: \\\\fwneld\\msdos\\RCS\\K\\fsc\\prj\\verlof\\src\\register.py,v 1.8 2003-01-27 14:06:10+01 moene Exp $
#
# $Log: register.py,v $
# Revision 1.8  2003-01-27 14:06:10+01  moene
# ignores calls with empty formfield data;
# prevents entering duplicate forms in succession (sessionID in person table).
#
# Revision 1.7	2003/01/14 09:48:31  moene
# added mailfrom in address list to send to:
#	server.sendmail( mailfrom, [ mailto, mailfrom ], mesg )
#
# Revision 1.6	2003/01/08 11:20:42  moene
# added sending error notification e-mail to application administrator.
#
# Revision 1.5	2003/01/07 11:47:36  moene
# code cleanup:
# exception handling code and messages;
# changed pattern to obtain surname in findPerson()
#
# Revision 1.4	2003/01/06 11:27:11  moene
# added sending e-mail from register.py.
#
# Revision 1.3	2002-12-17 16:55:10+01	moene
# corrected rejecting entries for users *with* a database account (Bug 7):
# moved raise NoNameError to within if t == ():
#
# Revision 1.2	2002-12-12 16:02:21+01	moene
# added flag for autocreate/no autocreate account.
# SQL quoting of quotes in remarks.
#
# Revision 1.1	2002/05/13 09:20:04  moene
# Initial revision
#
#

"""
Register.py - CGI program to register a verlof entry and send e-mail.

Take data from register.html form and add it to the database.
There are two tables: person and verlof. Person holds names,
e-mail addresses and session IDs, verlof holds verlof data and
a link (personID) to the person's entry in the person table.

Register first looks for the person's name in the person table
and it takes the personID to add in the new verlof entry in the
verlof table.

The format of a name is:
   surname-initials prefixes

For example:
   Moene-MJ (M.J. Moene)

   Eijk-van der Linden-JIF van der  (J.I.F. van der Eijk-van der Linden)
   \_________________/ \_/ \_____/
     surname|	initials|     |prefixes

Register looks for a persons name in three steps:
1. try to get an exact match
2. try to get a prefix match
3. try to match the surname only

This behaviour makes register.py forgiving in the way someone
specifies his name, allowing for small changes over time.
However, see below.

If the session ID of this entry is the same as the session ID for
the last entry made, register.py silently ignores the new entry.
If the session IDs differ, the session ID and e-mail address are updated.

When someone makes a verlof-entry for the first time, register.py
will create an entry in the person table, UNLESS
- automatic account creation is disabled, or
- an entry is found that has the same surname (see the three steps above)

If the unless clause applies, the person needs the assistance of
the database administrator to get an entry in the person table.

If the database entry was succesfully made, an e-mail is sent to the
'mailto' and 'mailfrom' e-mail addresses.
"""

#import cgitb; cgitb.enable()		     # debug

import os, sys, types, operator, string, re, zlib, smtplib, cgi, MySQLdb, config;
from mx import DateTime;

#
# Errors/traceback to Webserver error log file.
#
#sys.stderr = sys.stdout		     # make error messages visible in browser

global form, map

NoNameError	  = 'Exception_NoName'
AmbNameError	  = 'Exception_AmbName'
DupFormError	  = 'Exception_DuplicateFormError'
mailTemplateError = 'Exception_mailTemplateError'
mailSendError	  = 'Exception_mailSendError'
dbError 	  = 'Exception_dbError'

form = cgi.FieldStorage()

#
# define a map of known form fields to be able to
# discriminate between [form-field] and [other] in e-mail message:
#
# Note on default value for part (dag):
# if the vm/nm indicator entry field is disabled as a result of the
# hours value, some browsers, for example IE4, do not send the part field.
# However we do not want the part value to be empty, but show 'dag' instead
# in the X-info and X-part e-mail headers.
#

map  = {
   "dbhost"	 : "",	# hidden form fields
   "db" 	 : "",
   "mailto"	 : "",
   "name_srt"	 : "",
   "cat_txt"	 : "",
   "date1_dot"	 : "",
   "date2_dot"	 : "",
   "date1_inf"	 : "",
   "date2_inf"	 : "",
   "date1_txt"	 : "",
   "date2_txt"	 : "",
   "part_txt"	 : "",
   "remarks_flg" : "",
   "remarks_inf" : "",

   "name"	 : "",	# fill-in form fields
   "email"	 : "",
   "cat"	 : "",
   "date1"	 : "",
   "date2"	 : "",
   "hours"	 : "",
   "part"	 : "dag",
   "remarks"	 : ""
}

#
# class dB - register's interface to the mySQL database.
#

class dB:

   """Interface to mySQL verlof database."""

   def __init__(self, hostname, dbname):

      """Constructor"""

      self.dbname = dbname;
      self.open( hostname, dbname )


   def __del__(self):

      """Destructor"""

      try:
	 self.cursor.close()
	 self.db.close()
      except: pass


   def open( self, hostname, dbname ):

      """Open database by name."""

      self.db = MySQLdb.connect( host = hostname, db = dbname )
      self.cursor = self.db.cursor()


   def findPerson( self, name ):

      """Lookup entry for name. First try exact match, then a prefix match,
      then with initials stripped. To be more precise: the part right of
      rightmost dash; this also may be the second part of the surname!"""

      surname = re.sub(r'(?i)-[\w\ ]+$', '', name)

      # look for exact match(name-ab):

      count = self.cursor.execute("SELECT personID, email, sessionID FROM person WHERE name RLIKE '^%s$'" % (name) )

      # if none found, look for prefix match with initials (name-ab):

      if count == 0:
	 count = self.cursor.execute("SELECT personID, email, sessionID FROM person WHERE name RLIKE '^%s'" % (name) )

      # if none found, look for name only (name):

      if count == 0:
	 count = self.cursor.execute("SELECT personID, email, sessionID FROM person WHERE name RLIKE '^%s'" % (surname) )

      # if none found, return empty tuple:

      if count == 0:
	 return ()

      # if ambiguous, raise exception:

      if count > 1:
	 raise AmbNameError, name

      return self.cursor.fetchone()


   def getPersonID( self, name, email, SID ):

      """Obtain person ID for given name, creating an entry if no one found and
      automatic account creation is disabled. Also check if session ID SID differs
      from the last one. See also addEntry()."""

      t = self.findPerson( name )

      # if not found:
      #    config.mkaccount(0): raise NoNameError,
      #    config.mkaccount(1): insert person entry:

      if t == ():

	 if not config.mkaccount:
	    raise NoNameError, name

	 self.cursor.execute("INSERT INTO person(name, email, sessionID) VALUES('%s', '%s', '%s')" % (name, email, SID - 1) )
	 t = self.findPerson( name )

      pid, em, lastsid = t

      # if session ID is same as last one, raise DupFormError:

      if SID == lastsid:
	raise DupFormError

      # update SID and email:

      self.cursor.execute("UPDATE person SET email='%s', sessionID='%s' WHERE personID='%s'" % (email, SID, str(pid)) )

      return pid


   def addData( self, id, date1, date2, hours, category, part = None, remark = None ):

      """Add entry data into database.
      See also addEntry()."""

      qs = "INSERT INTO verlof SET personID='%s', date1='%s', date2='%s', hours=%s, category='%s'" \
					  % ( id, date1,      date2,	  hours,    category )

      parts = { 'vm':'vm', 'nm':'nm' }
      if part != None and parts.has_key( part.strip() ):
	 qs += ", part = '%s'" % part

      if remark != None and '' != remark.strip():
	 qs += ", remark = '%s'" % remark.replace( "\'", "\\'" ).replace( '\"', '\\"' )

      self.cursor.execute( qs )


   def addEntry( self, name, email, date1, date2, hours, part, category, remark, SID ):

      """Look for name, optionally creating an entry for it and enter data into database.

      name	'Leiden-JJP van'
      email	'Joan van Leiden <joan.van_Leiden@eld.fwn.LeidenUniv.nl>'
      date1	'2002-03-22'
      date2	'2002-03-22'
      hours	'6.5'
      part	'dag', 'vm', 'nm'; something other than 'vm' or 'nm' enters a NULL in the database
      category	mapping is as follows:
		'v'   verlof
		'bv'  bijzonder verlof
		'sv'  stuwmeer verlof
		'a'   ADV
		'va'  verplicht ADV
		't'   tegoed
		'at'  ADV tegoed
		'tr'  transport
      remark	optional remark; an empty remarks enters a NULL in the database.
      """

      pid = self.getPersonID( name, email, SID )
      self.addData( pid, date1, date2, hours, part, category, remark )


#
# global functions:
#

def _mkSessionID( m ):
   """Return session ID calculated from concatenated form data"""
   return zlib.crc32( reduce( operator.concat, m.values() ) )


def _addTodB():

   """Add formdata into database."""

#      cgi.test()

   db = dB( map['dbhost'], map['db'] )

   db.addEntry( map['name_srt'],
		     map['email'],
		     map['date1_inf'],
		     map['date2_inf'],
		     map['hours'],
		     map['cat'],
		     map['part'],
		     map['remarks'],
		     _mkSessionID( map ) )


def _sendmail():

   """Edit template e-mail message and send it to manager and user."""

   #
   # open and read e-mail message template:
   #

   try:
      file = open( 'email.txt', 'r' )
      mesg = file.read()
   except IOError:
      raise mailTemplateError
   else:
      file.close()

   #
   # edit message template, replacing [form-field] and [$environment-variable] with contents:
   #

   for key, value in map.items():
      mesg = mesg.replace( '[' + key + ']', value )

   while 1:
      matchobj = re.search( '\[\$' + r'(\w+)' + '\]', mesg )

      if matchobj == None:
	 break

      mesg = mesg.replace( matchobj.group(), os.environ.get( matchobj.group(1), '' ) )

   #
   # send the e-mail:
   #

   mailfrom, mailto = map['email'], map['mailto']

   try:
      server = smtplib.SMTP( config.smtphost )
      server.sendmail( mailfrom, [ mailto, mailfrom ], mesg )
   except:
      raise mailSendError
   else:
      server.quit()


def _register():

   """Add the verlof entry to the database and send the e-mail."""

   #
   # first print HTTP header:
   #

   print "Content-Type: text/html"	# HTML is following
#   print "Cache-Control: no-cache"
#   print "Pragma: no-cache"
#   print "Expires: 0"
   print				# blank line, end of headers
   print				# non-empty response

   #
   # check if called from a real form entry:
   #

   if form.keys() == []:
      return

   #
   # create normal dictionary from CGI form,
   # handle missing keys by initializing map (see top):
   #

   global map;

   for key in form.keys():
      map[key] = form[key].value

   #
   # add information to database and send e-mail; handle exceptions:
   #

   dbname = map['db']

   try:
      _addTodB()
      _sendmail()

   except NoNameError, name:
      _noNameAlert( name, dbname )

   except AmbNameError, name:
      _ambNameAlert( name, dbname )

   except DupFormError:
      pass
#      _dupFormAlert( DupFormError )

   except dbError:
      _dbErrorMsg( dbname )

   except mailTemplateError:
      _mailErrorMsg( mailTemplateError )

   except mailSendError:
      _mailErrorMsg( mailSendError )

   except:
      _otherErrorMsg()

   else:
      _mailSentAlert( form['mailto'].value, form['email'].value )


#
# global alert and error message functions:
#

def _noNameAlert( name, dbname ):
   """Report that specified name cannot be found."""

   _alert( """register: naam \'%s\' is niet gevonden in database \'%s\'

Uw naam is (nog) niet geregistreerd.
Uw verlofinformatie is niet opgenomen in de database.

Neem contact op met uw diensthoofd om uw naam toe te laten voegen."""  %  ( name, dbname ) )


def _ambNameAlert( name, dbname ):
   """Report that specified name is ambiguous."""

   _alert( """register: naam \'%s\' is ambigu in database \'%s\'

Uw verlofinformatie is niet opgenomen in de database.

Er is meer dan een naam in de database die lijkt op uw naam, mogelijk doordat:
1. uw opgave correspondeert met het voorste deel van een andere naam, OF
2. u geen initialen heeft opgegeven bij uw naam, OF
3. u uw naam anders heeft opgegeven dan in de database is vermeld en
   ik probeerde uw naam te vinden met weglating van de initialen.

Ik doe dit op deze manier, zodat kleine veranderingen in de manier
waarop u uw naam opgeeft zijn toegestaan. De keerzijde is, dat u geen
informatie kunt toevoegen, als uw naam op geschetste wijze overeenkomt
met die van iemand anders in de database.

Helaas kan dit voor u het geval zijn.

Neem hierover contact op met uw diensthoofd."""  %  ( name, dbname ) )


#   _alert( """%s: name \'%s\' is ambiguous in database \'%s\'
#
#Your entry has not been added to the database.
#
#More than one entry matches your name, probably because:
#1. your name is a prefix of someone elses name, OR
#2. you did not specify initials with your name, OR
#3. you specified your name different from before and
#   I tried to find your name without initials.
#
#I behave in this manner to be forgiving in the way you specify
#your name, allowing small changes over time. However, if your name
#clashes with someone elses already in the database, you are not able
#to create an entry for yourself.
#
#Unfortunately, this may be the case for you.
#
#Please contact the database administrator to obtain your personal name entry."""  %  ('register', detail, self.dbname) )


def _dupFormAlert( exception ):
   """Report that the information could not be entered in the database
   and no e-mail has been sent."""

   _error(exception)


def _dbErrorMsg( dbname ):
   """Report that the information could not be entered in the database
   and no e-mail has been sent."""

   _error("""Er is een fout opgetreden bij de verwerking van het formulier.

De gegevens zijn niet opgenomen in database %s en er is geen e-mail verstuurd.""" % dbname )


def _mailErrorMsg( exception ):
   """Report that the e-mail could not be sent, but that information
   has been entered in the database."""

   _error("""Er is een fout opgetreden bij de verwerking van het formulier (%s).

De gegevens zijn wel opgenomen in de database, maar er is geen e-mail verstuurd.""" % exception )


def _otherErrorMsg():
   """Report that something went wrong and that as a result the information supplied
   could not be processed."""

   _error("""register.py - probleem met verwerking

De door u verstrekte informatie kan niet worden verwerkt, omdat er
een fout optreedt als gevolg van uw verzoek. De fout is opgenomen in
de log-files en er zal zo spoedig mogelijk aandacht aan worden gegeven.""" )


def _mailSentAlert( mailto, mailfrom ):
   """Report that the e-mail has been sent."""

   _alert("""Het formulier is verzonden aan: \\n\\n%s\\n\\n%s""" % ( mailto, mailfrom ) )


def _alert(msg):
   """Create HTML response with a JavaScript alert."""

   sys.stdout.write( """<html><body>
<script type="text/javascript" language="JavaScript">
alert( "%s" );
</script>
</body></html>""" % msg.replace( '\n', '\\n' ) )


def _error(msg):
   """Create HTLM response with a JavaScript alert and
   print a traceback to the Webserver error_log (sys.stderr).

   Optionally an e-mail is sent to the application administrator as
   configured through ./configure."""

   _alert(msg)

   import traceback
   sys.stderr.write( 'register.py: error occurred, see traceback:\n' )
   traceback.print_exc( file=sys.stderr )

   _mailAdmin()

   sys.exit(0)


def _mailAdmin():
   """Send e-mail message with form and environment contents and traceback to administrator.

   config.appadmin holds the application administrator e-mail address,
   config.mailcmd specifies the mail program commandline.

   mail is only sent if both appadmin and mailcmd are non-empty."""

   if not config.appadmin or not config.mailcmd:
      return

   try:
      import traceback
      pipe=os.popen( config.mailcmd, 'w' )
      pipe.write   ( 'Error while executing register.py\n\n'  \
		     'Parameters:\n'  + `form`	     + '\n\n' \
		     'Environment:\n' + `os.environ` + '\n\n' )
      traceback.print_exc(file=pipe)
      pipe.close()
   except:
      pass


#
# main line:
#

_register()

#
# End of file
#

