#!/bin/env ksh
#
# mkdb.ksh - create verlof database.
#
# Verlof Web application, Copyright (C) 2002 by Universiteit Leiden.
#
# $Revision: 1.2 $
# $Date: 2002-05-21 10:12:48+02 $
# $Source: \\\\fwneld\\msdos\\RCS\\K\\fsc\\prj\\verlof\\src\\script\\mkdb.ksh,v $
# %Author: M.J. Moene %

#
# $Header: \\\\fwneld\\msdos\\RCS\\K\\fsc\\prj\\verlof\\src\\script\\mkdb.ksh,v 1.2 2002-05-21 10:12:48+02 moene Exp $
#
# $Log: mkdb.ksh,v $
# Revision 1.2  2002-05-21 10:12:48+02  moene
# changed dbuser="@localhost" to dbuser="''@localhost".
#
# Revision 1.1	2002/05/13 09:20:02  moene
# Initial revision
#
#

typeset dbserver="localhost" dbuser="''@localhost"
typeset     user="root"

function usage #
{
   print - "${prgname}	Version ${version}  Create verlof database."
   print - "Usage: ${prgname} [options] [-o output] [-s server] [--] [-] [database...]"
   print - "-h: show usage"
   print - "-d: print debugging information"
   print - "-i: select arguments interactively"
   print - "-v: verbose; print arguments"
   print - "-o output: output file or device"
   print - "-s server: mySQL server [localhost]"
   print - "-u user: user@host access [''@localhost]"
   print - "--: terminate options section"
   print - "-: process standard input"
   print - "database: verlof database to create"
}

function mkdb # ( dbname )
{
   typeset dbname="$1"

   cat <<__EOF
${prgname}  Version ${version}	Create verlof database.

Setting up verlof database with tables person and verlof
on server ${dbserver}.

SELECT, INSERT and UPDATE access is granted to users
connecting as '${dbuser}'.

__EOF

   typeset defdbname="$1"
   read dbname?"database name [${defdbname}]: " 	# request and read user's response

   case "${dbname}" in					# dispatch on response
      "") dbname=${defdbname};; 			#
   esac

   # request and read user's response

   read pw?"Please enter password for ${user} for mySQL on server ${dbserver}: "

   # create ELD-verlof database:

   print - "\nDeleting database ${dbname}: "
   /usr/local/mysql/bin/mysql	   --host=${dbserver} --user=root --password=${pw} <<__EOF
   USE ${dbname};
   REVOKE ALL PRIVILEGES
      ON *
    FROM ${dbuser};
__EOF
   /usr/local/mysql/bin/mysqladmin --host=${dbserver} --user=${user} --password=${pw} DROP ${dbname}

   print - "Creating database ${dbname}: "
   /usr/local/mysql/bin/mysqladmin --host=${dbserver} --user=${user} --password=${pw} create ${dbname}

   # create tables for verlof database:

   print - "Creating tables person, verlof in database ${dbname}: "
   /usr/local/mysql/bin/mysql --host=${dbserver} --user=root --password=${pw} ${dbname} <mktbl.sql

   # grant privileges to unspecified/anonymous user:

   /usr/local/mysql/bin/mysql --host=${dbserver} --user=${user} --password=${pw} <<__EOF
   USE ${dbname};
   GRANT SELECT,INSERT,UPDATE
      ON *
      TO ${dbuser};
__EOF

   # show the result:
   /usr/local/mysql/bin/mysqlshow --host=${dbserver} --user=${user} --password=${pw} ${dbname}
   /usr/local/mysql/bin/mysql	  --host=${dbserver} --user=${user} --password=${pw} <<__EOF
   USE mysql;
   SELECT * from db;
__EOF

}

#
# version:
#

typeset version="1.0"

#
# definitions:
#

integer UNIX=${ROOTDIR:-1}			# determine OS

if [ UNIX -ne 0 ]; then
   typeset BIN="/bin"				# Unix bin
else
   typeset BIN="${ROOTDIR}/bin" 		# MKS Toolkit bin directory
fi

typeset AWK="${BIN}/awk"			# AWK program

#
# main line:
#

trap "print "Interrupted."; exit 2" INT 	# action on ^C

typeset prg="$0" prgname="$(basename $0 .ksh)"	# save script name

integer optInteractive=0			# interactive flag
integer       optDebug=0			# debug flag
integer     optVerbose=0			# verbose flag
typeset     outputName=""			# output name

#
# Test on argument content instead of $#, because the use of "%1" etc. in
# batch file leads to empty arguments in this Korn shell script.
#

if [ "X$1" = "X" ]				# any arguments given?
then
   usage					# no : show usage
   exit 1;
fi

#
# Preprocessing:							# <<====
#


#
# Process arguments specified:
#

typeset arg="$1"				# initialize arg

while true					# loop while options present
do
   case "$arg" in
      -i*) optInteractive=1 ;;				# -i: interactive
      -d*) optDebug=1	    ;;				# -d: debug
      -v*) optVerbose=1     ;;				# -v: verbose

      -s*) if [ "X$2" = "X" ]; then
	      print - "${prgname}: argument expected for option -s." ;
	      exit 1 ;
	   fi
	   dbserver="$2"; shift ;;

      -u*) if [ "X$2" = "X" ]; then
	      print - "${prgname}: argument expected for option -w." ;
	      exit 1 ;
	   fi
	   dbuser="$2"; shift ;;

      -o*) if [ "X$2" = "X" ]; then
	      print - "${prgname}: argument expected for option -o." ;
	      exit 1 ;
	   fi
	   outputName="$2"; shift ;;

      --*) break	    ;;				# --: end of options section
       -*) print - "${prgname}: unknown option '"$arg"'.\n"	# -?: invalid option
	   usage;
	   exit 1 ;;
	*) break ;;					# ? : no more options
   esac

   arg="${arg#-?}"					# compute next arg
   case "$arg" in
      "") shift; arg="$1" ;;				# next argument
       *) arg="-$arg" ;;				# next option within argument
   esac
done

#
# show current values (DEBUG):
#

if [ optDebug -ne 0 ]; then
   print -n "OS: "					# OS
   if [ UNIX -ne 0 ]; then
      print "Unix"
   else
      print "MS-DOS"
   fi

   print "Option: interactive:${optInteractive}"	# interactive
   print "Option: debug:${optDebug}"			# ver
   print "Option: verbose:${optVerbose}"		# verbose
   print "Option: outputName:${outputName}"		# output name
fi

#
# Process arguments specified:
#

typeset response=""					# response for interactive

while [ "X$1" != "X" ]					# while any argument left
do
   #
   # Process current argument:
   #

   if [ optDebug -ne 0 ]; then				# print current argument (DEBUG)
      print "Argument: ${1}"
   fi

   #
   # Interactive?
   #

   if [ optInteractive -ne 0 ]; then
      read response?"${1} (Y/n/q)? "			# request and read user's response
   fi

   case "${response}" in				# dispatch on response
      ""|[yY]) ;;					# yes : handle this one
	 [nN]) shift; continue ;;			# no  : skip this one
	 [qQ]) print - "Terminated."; break ;;		# quit: skip rest: terminate processing
   esac

   #
   # Inform user, if verbose:
   #

   if [ optVerbose -ne 0 ]; then
      print "${1}:"					# print argument
   fi

   #
   # Now do the real processing...					# <<====
   #

   mkdb ${1}

   #
   # Select next argument:
   #

   shift
done

#
# Postprocessing:							# <<====
#


#
# End of file
#

