#!/usr/local/bin/python
#
# view.py - CGI program to view verlof entries.
#
# Verlof Web application
#
# Copyright (c) 2002, by Leiden University.
# verlof is free software.  See the file Copying for copying permission.
#
# $Revision: 1.6 $
# $Date: 2003-01-29 09:19:19+01 $
# $Source: \\\\fwneld\\msdos\\RCS\\K\\fsc\\prj\\verlof\\src\\view.py,v $
# %Author: M.J. Moene %

#
# $Header: \\\\fwneld\\msdos\\RCS\\K\\fsc\\prj\\verlof\\src\\view.py,v 1.6 2003-01-29 09:19:19+01 moene Exp $
#
# $Log: view.py,v $
# Revision 1.6  2003-01-29 09:19:19+01  moene
# <>
#
# Revision 1.5	2003/01/27 12:16:28  moene
# ignores calls with empty formfield data.
#
# Revision 1.4	2003/01/08 11:21:59  moene
# added sending error notification e-mail to application administrator.
# moved printing of HTTP header.
#
# Revision 1.3	2002/05/21 11:42:59  moene
# changed YearView.mkDayCat():
# added yr parameter.
# allowed to override Dies Natalis (8 feb).
# now show category for zero hours.
# transport, tegoed and ADV tegoed not tagged anymore.
#
# Revision 1.2	2002/05/17 14:25:02  moene
# changed class YearView:
# removed stuwmeer verlof ('sv') from query_yearout() and query_monthout().
# for negative hours, category is removed again from days if present and
# the same as the previously recorded category.
# added Leidens Ontzet (3 oktober) to mkDayCat().
#
# Revision 1.1	2002/05/13 09:20:05  moene
# Initial revision
#
#

"""
View.py - CGI program to view verlof entries.


"""

#import cgitb; cgitb.enable()		 # debugging

import os, sys, string, re, cgi, MySQLdb, config;
from mx import DateTime;
from mx.DateTime import Feasts;

#
# Errors/traceback to Webserver error log file.
#
#sys.stderr = sys.stdout		     # make error messages visible in browser

global form
form = cgi.FieldStorage()


# ------------------------------------------------------------------------
# class _View
# ------------------------------------------------------------------------

class _View:

   """Base class for various page views"""

   def __init__(self, hostname, dbname, year, name):

      """Constructor"""

      self.hostname = hostname
      self.dbname   = dbname
      self.year     = year or ''
      self.name     = name or ''
      self.yrlist   = year or ''
      self.nmlist   = name or ''

      self.open( hostname, dbname )

      self.NoDataError	= 'Exception_NoData'
      self.NoNameError	= 'Exception_NoName'
      self.AmbNameError = 'Exception_AmbName'
      self.PureVirtual	= 'Exception_PureVirtual'


   def __del__(self):

      """Destructor"""

      try:
	 self.cursor.close()
	 self.db.close()
      except: pass


   def open( self, hostname, dbname ):

      """Open database by name."""

      self.db = MySQLdb.connect( host = hostname, db = dbname )
      self.cursor = self.db.cursor()


   def mkInput(self):

      """Transform the year and name specification strings from the
      form to a list of years and a list of names to present in the view."""

      # format "crama-b, moene-mj overgauw-cf " to "crama-b,moene-mj,overgauw-cf",
      #   and transform strings to lists:

      self.yrlist = re.sub( '[ ,;:]+', ',', self.yrlist.strip()).split( ',' )
      self.nmlist = re.sub( '[ ,;:]+', ',', self.nmlist.strip()).split( ',' )

      # transform wildcards to empty lists

      if self.yrlist == [''] or self.yrlist == ['*']: self.yrlist = []
      if self.nmlist == [''] or self.nmlist == ['*']: self.nmlist = []

      # lookup specified years for which data exists,
      #   and lookup personID's for non-empty namelist:
      #
      # Note: a derived class can override this behaviour
      #       and simply return the given list, without
      #       searching the database.

      self.yrlist = self.query_years( self.yrlist )
      self.idlist = self.query_names( self.nmlist )

#      print 'mkInputList: yrlist: ', self.yrlist
#      print 'mkInputList: idlist: ', self.idlist


   def query(self):
      """Query database (abstract method)."""
      raise self.PureVirtual


   def query_names(self, nmlist):

      """Return a list of personID's for the names specified, using a
      prefix match and sorted by name. If an empty list of names is
      specified, query the database for all names."""

#      print 'query_names: ', nmlist

      qs = "SELECT personID FROM person %s ORDER BY name"

      if not nmlist:
	 if self.cursor.execute( qs % '' ) == 0:
	    return []
	 else:
	    return [ x[0] for x in self.cursor.fetchall() ]

      else:
	 WHERE = " WHERE name RLIKE '^%s'"
	 l = []
	 for n in nmlist:
	    count = self.cursor.execute( qs % WHERE % n )
	    if count == 0:
	       raise self.NoNameError, n
	    elif count > 1:
	       raise self.AmbNameError, n
	    elif count == 1:
	       l.append( self.cursor.fetchone()[0] )

	 return l


   def query_years(self, yrlist):

      """Return the list of years that are available in the database of
      the years specified. If an empty list of years is specified, return
      all years available in the database."""

#      print 'query_years: ', yrlist

      qs = "SELECT YEAR(date1) FROM verlof %s GROUP BY YEAR(date1)"

      if not yrlist:
	 if self.cursor.execute( qs % '' ) == 0:
	    return []
	 else:
	    return [ x[0] for x in self.cursor.fetchall() ]

      else:
	 WHERE = "WHERE YEAR(date1)=%s"
	 l = []
	 for y in yrlist:
	    if self.cursor.execute( qs % WHERE % y ) > 0:
	       l.append( self.cursor.fetchone()[0] )

	 return l


   def mkView(self):
      """Create a complete view (abstract method)."""
      raise self.PureVirtual


   def mkTable(self):
      """Create a table with the information collected (abstract method)."""
      raise self.PureVirtual


   def mkEntries(self):
      """Create table rows for the information collected (abstract method)."""
      raise self.PureVirtual


   def mkEntry(self, row):
      """Create a single table row for the information collected (abstract method)."""
      raise self.PureVirtual


   def mkNoNameTable(self, n):

      """Create a table with a 'name not found' error message."""

      self.mkTableHead()
      self.mkNoNameEntry(n)
      self.mkTableTail()


   def mkNoNameEntry(self, n):

      """Create a table row with a 'name not found' error message."""

      print "<tr><td class=error>&nbsp;Naam '%s' niet gevonden.&nbsp;</td></tr>" % n


   def mkAmbNameTable(self, n):

      """Create a table with an 'ambiguous name' error message."""

      self.mkTableHead()
      self.mkAmbNameEntry(n)
      self.mkTableTail()


   def mkAmbNameEntry(self, n):

      """Create a table row with an 'ambiguous name' error message."""

      self.data = self.cursor.fetchall()
      print "<tr><td class=error>&nbsp;Naam '%s' is ambigu:&nbsp;</td></tr>" % n

      for id in self.data:
	 self.cursor.execute("""
	    SELECT name
	      FROM person
	     WHERE personID=%s""" % id )

	 print "<tr><td>&nbsp;'%s'&nbsp;</td></tr>" % self.cursor.fetchone()


   def mkNoDataTable(self, pair):

      """Create a table with a 'no data available' error message."""

      self.mkTableHead()
      self.mkNoDataEntry(pair)
      self.mkTableTail()


   def mkNoDataEntry(self, pair):

      """Create a table row with a 'no data available' error message."""

      y,id = pair
      self.cursor.execute("""
	 SELECT name
	   FROM person
	  WHERE personID=%s""" % id )

      n = self.cursor.fetchone()[0]
      print "<tr><td colspan=5 class=error>&nbsp;%s: geen gegevens van %s.&nbsp;</td></tr>" % (n,y)


   def mkTableHead(self):
      """Start an HTML form/table."""
      print """<form><table border=0 cellspacing=0 Xcellpadding=0>"""


   def mkTableTail(self):
      """End an HTML form/table."""
      print '</table></form>'


   def mkHead(self):
      """Start an HTML document."""
      print '<html>\n<head>'
      self.mkStyle()
      self.mkScript()
      print '</head>\n<body>'


   def mkTail(self):
      """End an HTML document."""
      print '</body></html>'


   def mkStyle(self):
      """Create a CSS section."""
#      print """<link rel="STYLESHEET" href="css/default.css" type="text/css">"""
      print """<link rel="STYLESHEET" href="css/view.css" type="text/css">"""
      self.mkStyleHead()
      self.mkStyleBody()
      self.mkStyleTail()


   def mkStyleHead(self):
      """Start a CSS section."""
      print '<style type="text/css">'


   def mkStyleTail(self):
      """End a CSS section."""
      print '</style>'


   def mkStyleBody(self):
      """Create a CSS section's contents."""
      pass


   def mkScript(self):
      """Create a JavaScript section."""
      self.mkScriptHead()
      self.mkScriptBody()
      self.mkScriptTail()


   def mkScriptHead(self):
      """Start a Javacript section."""
      print '<script type="text/javascript" language="JavaScript1.3">'


   def mkScriptTail(self):
      """End a JavaScript section."""
      print '</script>'


   def mkScriptBody(self):
      pass


   def catdesc( self, cat ):

      """Return description for database category code."""

      catdict = {
	  'v': 'verlof',
	 'sv': 'stuwmeer verlof',
	 'bv': 'bijzonder verlof',
	  'a': 'ADV',
	 'va': 'verplicht ADV',
	  't': 'tegoed',
	 'at': 'ADV tegoed',
	 'tr': 'transport',
	 'na': 'niet aanwezig' }

      return catdict[ cat ]


# ------------------------------------------------------------------------
# class TotalView - present a year summary
# ------------------------------------------------------------------------

class TotalView(_View):

   """Class for year summary view."""

   def __init__(self, hostname, dbname, year, name ):
       _View.__init__(self, hostname, dbname, year, name )


   def mkView(self):

      """Present a year summary.

      For each year specified, or available in database, and
      for each name specified, or available in database make a
      summary of hours available and hours spent."""

      self.mkHead()

      # collect year and name info:

      try:
	 self.mkInput()

      except self.NoNameError, detail:
	 self.mkTableSep()
	 self.mkNoNameTable(detail)

      except self.AmbNameError, detail:
	 self.mkTableSep()
	 self.mkAmbNameTable(detail)

      else:
	 # traverse years, names

	 for id in self.idlist:
	    for y in self.yrlist:
	       try:
		  self.query( (y, id) )
		  self.mkTableSep()
		  self.mkTable()

	       except self.NoDataError:
		  self.mkTableSep()
		  self.mkNoDataTable( (y, id) )

      self.mkTail()


   def query(self, pair):

      """Query database for a SUM of hours for each category
      for specified year and name (pair). Return a row with
      year, name, category and hour information."""

      qs = """
	 SELECT YEAR(date1), name, category, SUM(hours)
	   FROM verlof, person
	  WHERE YEAR(date1)=%s
	    AND verlof.personID=person.personID
	    AND person.personID=%s
	  GROUP BY category"""

      if self.cursor.execute( qs % pair ) == 0:
	 raise self.NoDataError

      self.data = self.cursor.fetchall()


   def query_years(self, yrlist):

      """For wildcard, lookup available years in database,
      otherwise use list of specified years."""

      if yrlist: return yrlist
      else     : return _View.query_years( self, yrlist )


   def mkTable(self):

      """Create a summary table for one person, one year."""

      self.mkTableHead()
      self.mkEntries()
      self.mkTableTail()


   def mkEntries(self):

      """Create summary table rows for one person, one year."""

      cat = { 'tr':0, 't':0, 'v':0, 'sv':0, 'bv':0, 'at':0, 'va':0, 'a':0, 'na':0 }

      for row in self.data:
	 year, name, c, h = row
	 cat[c] = h

      total_v = cat['tr'] + cat[ 't'] - cat['v']  # without stuwmeer verlof
      total_a = cat['at'] - cat['va'] - cat['a']
      total_g = total_v + total_a

      sfmt = '&nbsp;&nbsp;%-25s'      # string format

      # Stuwmeer verlof displayed separately:

      tbldata = [
	 ( (  'colspan=5', '<hr >'   ), 				  ),
	 ( (  'class=head colspan=4', " %s" % name     ), ('class=head align=right',  "%d " % year )   ),
	 ( (  'colspan=5',   '' 	    ),					),
	 ( ('',''),(  'colspan=1', '  V e r l o f'	  ), ('',''), ('colspan=2',	      '  A D V')   ),
	 ( ('class=right', " %g" % cat['tr']), ('', sfmt % self.catdesc('tr')), ('class=right', "%g" % cat['at']), ('', sfmt % self.catdesc('at')), ('', ''), ),
	 ( ('class=right', " %g" % cat[ 't']), ('', sfmt % self.catdesc( 't')), ('class=right', "%g" % cat['va']), ('', sfmt % self.catdesc('va')), ('', ''), ),
	 ( ('class=right', " %g" % cat[ 'v']), ('', sfmt % self.catdesc( 'v')), ('class=right', "%g" % cat[ 'a']), ('', sfmt % self.catdesc( 'a')), ('', ''), ),
	 ( ('class=right', " <b>%g</b>" % total_v  ), ('',			  ''), ('class=right', "<b>%g</b>" % total_a  ), ('',			     ''), ('class=right', "te besteden	<b>%g</b> " % total_g  ) ),
	 ( (  'colspan=5',   '' 	   ),			     ),
	 ( ('class=right', " %g" % cat['sv']), ('colspan=4', sfmt % self.catdesc('sv')),  ),
	 ( ('class=right', " %g" % cat['bv']), ('colspan=1', sfmt % self.catdesc('bv')), ('class=right', " %g" % cat['na']), ('colspan=2', sfmt % self.catdesc('na')),	),
	 ( (  'colspan=5',   '' 	   ),			     ),
      ]

      for row in tbldata:
	 self.mkRow(row)


   def mkRow(self, row):

      """Create a single summary table row."""

      print '<tr>'
      for cell in row:
	 self.mkCell(cell)
      print '</tr>'


   def mkCell(self, cell):

      """Create a single summary table cell."""

      style, data = cell
      if ( '' == data ): data = '&nbsp;'
      data = string.replace( data, ' ', '&nbsp;')

      print "<td %s>%s</td>" % (style, data)


   def mkTableSep(self):
      """Create break between to tables."""
      print "<br><br>"


   def mkStyle(self):
      """Create CSS section's contents."""
      _View.mkStyle(self)
      print """<link rel="STYLESHEET" href="css/view-total.css" type="text/css">"""



# ------------------------------------------------------------------------
# class TotalViewCDF - total view as comma delimited file
# ------------------------------------------------------------------------

class TotalViewCDF(TotalView):

   """Class for year summary view, comma delimited output"""

   def __init__(self, hostname, dbname, year, name, sep ):
      TotalView.__init__(self, hostname, dbname, year, name )

      # define category sequence for output:

      self.sep = sep or ';'
      self.cat = ( 'tr', 't', 'v', 'sv', 'bv', 'at', 'va', 'a', 'na' )


   def mkHeading(self):

      """Print heading line."""

      str = "# naam; jaar"
      for c in self.cat:
	 str += "; %s" % self.catdesc(c)
      print str


   def mkEntries(self):

      """Print year summary row for one person, one year."""

      hrs = {}				# initialize dictionary for hour values
      for c in self.cat:
	 hrs[ c ] = 0

      for row in self.data:		# fill hours dictionary
	 year, name, c, h = row
	 hrs[c] = h

      str = '"%s"%s%d' % (name, self.sep, year)
      for c in self.cat:
	 str += '%s%d' % (self.sep, hrs[c])
      print str


   def mkNoNameTable(self, n):

      """Print a row with a 'name not found' error message."""

      print "Naam '%s' niet gevonden." % n


   def mkAmbNameTable(self, n):

      """Print a row with an 'ambiguous name' error message."""

      self.data = self.cursor.fetchall()
      print "Naam '%s' is ambigu:" % n

      for id in self.data:
	 self.cursor.execute("""
	    SELECT name
	      FROM person
	     WHERE personID=%s""" % id )

	 print "'%s'" % self.cursor.fetchone()


   def mkNoDataTable(self, pair):

      """Print a row with a 'no data available' error message."""

      y,id = pair
      self.cursor.execute("""
	 SELECT name
	   FROM person
	  WHERE personID=%s""" % id )

      n = self.cursor.fetchone()[0]
      print '"%s: geen gegevens van %s."' % (n,y)


   def mkTableHead(self):
      """(Start HTML table.)"""
      pass


   def mkTableTail(self):
      """(Terminate HTML table.)"""
      pass


   def mkTableSep(self):
      """(Create break between to tables.)"""
      pass


   def mkHead(self):
      """Start a Text document."""
      print "<pre>"
      self.mkHeading()


   def mkTail(self):
      """End text document."""
      print "</pre>"
      pass


# ------------------------------------------------------------------------
# class YearView -
# ------------------------------------------------------------------------

class YearView(_View):

   """Class for year view, one row per month."""

   def __init__(self, hostname, dbname, year, name ):
       _View.__init__(self, hostname, dbname, year, name )


   def mkView(self):

      """Present a year overview, one row per month.

      For each year specified, or available in database, and
      for each name specified, or available in database make a
      year overview of days used."""

      self.mkHead()

      # collect year and name info:

      try:
	 self.mkInput()

      except self.NoNameError, detail:
	 self.mkTableSep()
	 self.mkNoNameTable(detail)

      except self.AmbNameError, detail:
	 self.mkTableSep()
	 self.mkAmbNameTable(detail)

      else:
	 # traverse years, names

	 for id in self.idlist:
	    for y in self.yrlist:
	       try:
		  y = eval("%s" % y)	    # make sure y is a number
		  self.query( (y, id) )
		  self.mkDayCat(y)
		  self.mkTableSep()
		  self.mkTable()

	       except self.NoDataError:
		  self.mkTableSep()
		  self.mkNoDataTable( (y, id) )

      self.mkTail()


   def query(self, pair):

      """Query database for hours to spend this year, hours spent
      this year, hours spent each month and category for days spent
      for specified year and name (pair). Return a row with
      name, date1, date2, hour and category information."""

      self.query_yearin  ( pair )    # determine hours to spend (rest + new and new ADV)
      self.query_yearout ( pair )    # determine hours spent (ADV, verlof)
      self.query_monthout( pair )    # determine hours spent each month (ADV, verlof)

      qs = """
	 SELECT name, date1, date2, hours, category
	   FROM verlof, person
	  WHERE YEAR(date1)=%s
	    AND verlof.personID=person.personID
	    AND person.personID=%s
	  ORDER BY date1"""

      if self.cursor.execute( qs % pair ) == 0:
	 raise self.NoDataError

      self.data = self.cursor.fetchall()


   def query_yearin(self, pair):

      """Quey database for hours to spend (transport + new, new ADV)."""

      y, id = pair

      qs = """
	 SELECT SUM(hours)
	   FROM verlof
	  WHERE YEAR(date1)=%s
	    AND personID=%s
	    AND category IN (%s)"""

      if self.cursor.execute( qs % (y, id, "'tr', 't'" ) ) == 0:
	 raise self.NoDataError

      self.yearin_v = self.cursor.fetchone()[0]
      if not self.yearin_v:
	 self.yearin_v = 0

      if self.cursor.execute( qs % (y, id, "'at'" ) ) == 0:
	 raise self.NoDataError

      self.yearin_a = self.cursor.fetchone()[0]
      if not self.yearin_a:
	 self.yearin_a = 0


   def query_yearout(self, pair):

      """Quey database for hours spent (ADV, verlof)."""

      y, id = pair

      qs = """
	 SELECT SUM(hours)
	   FROM verlof
	  WHERE YEAR(date1)=%s
	    AND personID=%s
	    AND category IN (%s)"""

      if self.cursor.execute( qs % (y, id, "'v'") ) == 0:
	 raise self.NoDataError

      self.yearout_v = self.cursor.fetchone()[0]
      if not self.yearout_v:
	 self.yearout_v = 0

      if self.cursor.execute( qs % (y, id, "'a', 'va'") ) == 0:
	 raise self.NoDataError

      self.yearout_a = self.cursor.fetchone()[0]
      if not self.yearout_a:
	 self.yearout_a = 0


   def query_monthout(self, pair):

      """Query database for hours spent each month (ADV, verlof)."""

      y, id = pair

      qs = """
	 SELECT SUM(hours)
	   FROM verlof
	  WHERE YEAR(date1)=%s
	    AND MONTH(date1)=%s
	    AND personID=%s
	    AND category IN (%s)"""

      self.monthout_a = {}  # initialize monthly ADV spent
      self.monthout_v = {}  # initialize monthly verlof spent

      for m in range(1, 13):
	 self.monthout_v[ m ] = 0
	 if self.cursor.execute( qs % (y, m, id, "'v'") ) > 0:
	    self.monthout_v[ m ] = self.cursor.fetchone()[0]

	 self.monthout_a[ m ] = 0
	 if self.cursor.execute( qs % (y, m, id, "'a', 'va'") ) > 0:
	    self.monthout_a[ m ] = self.cursor.fetchone()[0]


   def query_years(self, yrlist):

      """For wildcard, lookup available years in database,
      otherwise use list of specified years."""

      if yrlist: return yrlist
      else     : return _View.query_years( self, yrlist )


   def mkDayCat(self, yr):

      """Determine the verlof category for each day in the year
      for which verlof has been specified. Additionally indicate
      feasts like Easter, koninginnedag etc.."""

      self.dayCat = {}	  # create empty category array/dictionary

      #
      # feasts allowed to override:
      # Dies Natalis
      #

      self.dayCat[ DateTime.Date(yr, 2,  8).day_of_year ] = 'zo' # Dies Natalis Universiteit

      #
      # days specified:
      #

      for row in self.data:
	 nm, d1, d2, hrs, cat = row
#	  print nm, d1.strftime('%d %b %Y'), d1.day_of_year, d2.strftime('%d %b %Y'), d2.day_of_year, cat, hrs,

	 # (un)tag all days in specified range:
	 #
	 # hrs >=  0: add category
	 # hrs <   0: remove category if it exists and is the same
	 #

	 if not cat in ('tr', 't', 'at'):
	    for n in range(d1.day_of_year, d2.day_of_year + 1 ):
	       if hrs >= 0:
		  self.dayCat[n] = cat
	       elif hrs < 0 and self.dayCat.has_key(n) and self.dayCat[n] == cat:
		  del self.dayCat[n]

      #
      # feasts:
      #

      self.dayCat[ DateTime.Date(yr, 1,  1).day_of_year ] = 'zo' # Nieuwjaarsdag
      self.dayCat[ DateTime.Date(yr, 4, 30).day_of_year ] = 'zo' # Koninginnedag
      self.dayCat[ DateTime.Date(yr, 5,  5).day_of_year ] = 'zo' # Bevrijdingsdag
      self.dayCat[ DateTime.Date(yr,10,  3).day_of_year ] = 'zo' # Leidens Ontzet

      self.dayCat[ Feasts.GoodFriday   (yr).day_of_year ] = 'zo' # Goede vrijdag
      self.dayCat[ Feasts.EasterSunday (yr).day_of_year ] = 'zo' # 1e Paasdag
      self.dayCat[ Feasts.EasterMonday (yr).day_of_year ] = 'zo' # 2e Paasdag
      self.dayCat[ Feasts.Ascension    (yr).day_of_year ] = 'zo' # Hemelvaart
      self.dayCat[ Feasts.WhitSunday   (yr).day_of_year ] = 'zo' # 1e Pinksterdag
      self.dayCat[ Feasts.WhitMonday   (yr).day_of_year ] = 'zo' # 2e Pinksterdag
      self.dayCat[ DateTime.Date(yr,12, 25).day_of_year ] = 'zo' # 1e kerstdag
      self.dayCat[ DateTime.Date(yr,12, 26).day_of_year ] = 'zo' # 2e kerstdag

      self.curname = nm     # save current name and year
      self.curyear = yr     #	for use in mkEntries()


   def mkTable(self):

      """Create a year ADV/verlof overview displaying all dates for
      one person, one year. Color indicates the verlof category.

      Further the table shows the total number of hours to spend for
      ADV and verlof, the number of hours spent each month and the
      hours left to spent this year."""

      self.mkTableHead()
      self.mkEntries()
      self.mkTableTail()


   def mkEntries(self):

      """Print all table rows."""

      self.mkPersonHeading( self.curname, self.curyear )
      self.mkDayHeading   ( '%gA' % self.yearin_a, '%gV' % self.yearin_v )

      self.yearday = 0	 # keep track of day in year (incremented in mkDay() )

      for m in range(1,13):
	 self.mkMonth( self.curyear, m )

      self.mkLegendHeading( self.yearin_a - self.yearout_a, self.yearin_v - self.yearout_v )


   def mkPersonHeading(self, n, y):

      """Print name and year concerned."""

      print '<tr><td class=white colspan=27>&nbsp;</td><td colspan=13 class=namhead>%s&nbsp;&nbsp;&nbsp;%s&nbsp;</td></tr>' % (n, y)


   def mkDayHeading(self, a, v):

      """Print day heading and ADV and verlof hours to spend this year."""

      day = ( 'Ma', 'Di', 'Wo', 'Do', 'Vr', 'Za', 'Zo',)
      cls = 'class=dayhead'

      print "<tr><td %s>&nbsp;</td>" % cls,
      for n in range(37):
	 print "<td %s>%s</td>" % ( cls, day[n % 7]),

      print "<td %s>&nbsp;%s&nbsp;</td><td %s>&nbsp;%s&nbsp;</td></tr>" % (cls, a, cls, v)


   def mkLegendHeading( self, total_a, total_v ):

      """Print color attribute legend and ADV and verlof left to spend this year."""

      print """<tr>
      <td class=monhead>&nbsp;</td>
      <td colspan=7>&nbsp;</td>
      <td class="lv" >&#8226;</td><td class=legend colspan=3>%s</td>
      <td class="lsv">&#8226;</td><td class=legend colspan=5>%s</td>
      <td class="lbv">&#8226;</td><td class=legend colspan=5>%s</td>
      <td class="la" >&#8226;</td><td class=legend colspan=2>%s</td>
      <td class="lva">&#8226;</td><td class=legend colspan=4>%s</td>
      <td class="lna">&#8226;</td><td class=legend colspan=4>%s</td>
      <td colspan=1>&nbsp;</td>
      <td class=ttla>&nbsp;%3gA&nbsp;</td><td class=ttlv>&nbsp;%3gV&nbsp;</td>
      </tr>""" % (self.catdesc('v'), self.catdesc('sv'), self.catdesc('bv'),
		  self.catdesc('a'), self.catdesc('va'), self.catdesc('na'), total_a, total_v)


   def mkMonth(self, y, m):

      """Print row for this month."""

      mon = ( '', 'Jan', 'Feb', 'Mrt', 'Apr', 'Mei', 'Jun', 'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Dec' )

      d = DateTime.DateTime( y, m, 1 )
      w = d.day_of_week
      x = d.days_in_month

      print "<tr><td class=monhead>&nbsp;%s&nbsp;</td>" % mon[d.month],

      for n in range(37):
	 self.mkDay( n % 7, n - w + 1, x )

      self.mkMonTotal( 'ttla', self.monthout_a[m] )
      self.mkMonTotal( 'ttlv', self.monthout_v[m] )

      print "</tr>"


   def mkDay(self, wd, md, max ):

      """Print cell for this 'date' with category attribute as CSS class value."""

      cat = ''
      cls = ''

      # determine class for proper month days:

      if 0 < md <= max:
	 self.yearday += 1; yd = self.yearday
	 if self.dayCat.has_key( yd ):
	    cat = self.dayCat[ yd ]
	    cls = 'class=' + cat

      # don't indicate transport, and tegoed values:

      if cat in ('tr', 't', 'at'):
	 cls = ''

      # number cell contents for proper month day, empty otherwise:

      if 0 < md <= max:
	 num = ' %2d ' % md
      else:
	 cls = ''; num = ' '

      # set class for saturdays, sundays and days tagged 'zo':

      if 5 <= wd <= 6 or cat == 'zo':
	 cls = 'class=zo'

      # print the cell:

      num = num.replace( ' ', '&nbsp;' )

      print "<td %s>%s</td>" % (cls, num),


   def mkMonTotal( self, cat, hrs ):

      """Print ADV and verlof totals for this month."""

      # month total cell contents, or empty:

      if hrs:	    # None, <0, >0
	 num = (' %2d ' % hrs).replace( ' ', '&nbsp;' )
      else:
	 num = '    '

      # print the cell:

      num = num.replace( ' ', '&nbsp;' )

      print "<td %s>%s</td>" % ('class=' + cat, num),


   def mkTableSep(self):
      """Create break between to tables."""
      print "<br><br>"


   def mkStyle(self):
      """Create CSS section's contents."""
      _View.mkStyle(self)
      print """<link rel="STYLESHEET" href="css/view-year.css" type="text/css">"""


# ------------------------------------------------------------------------
# class ListView -
# ------------------------------------------------------------------------

class ListView(_View):

   """Class for a sorted list of verlof entries."""

   def __init__(self, hostname, dbname, year, name, sort, uri ):
       _View.__init__(self, hostname, dbname, year, name )
       self.uri  = uri
       self.sort = sort


   def mkView(self):

      """Present a sorted list of verlof entries.

      For each year specified, or available in database, and
      for each name specified, or available in database list the
      database contents."""

      self.mkHead()

      try:
	 self.mkInput()
	 self.mkTable()

      except self.NoNameError, detail:
	 self.mkNoNameTable(detail)

      except self.AmbNameError, detail:
	 self.mkAmbNameTable(detail)

      self.mkTail()


   def query(self):

      """Query database for verlof info for specified years and names.
      If no years and/or names are specified, information for all years
      and/or names is returned.

      Query returns rows with name, cetegory, date1, date2, hours, part
      and remark information. The sorting order is determined by the sort
      specification privided at construction."""

      YEAR = ''
      NMID = ''

#      if self.yrlist: print tuple( self.yrlist )
#      if self.idlist: print tuple( self.idlist )

      if self.yrlist: YEAR = 'AND YEAR(date1) in (%s)'	   % str( map( str, self.yrlist ) )[1:-1]
      if self.idlist: NMID = 'AND person.personID in (%s)' % str( map( str, self.idlist ) )[1:-1]

      qs = """
	 SELECT name, category, date1, date2, hours, part, remark
	   FROM person, verlof
	  WHERE verlof.personID=person.personID
	     %s
	     %s
	     %s""" % ( YEAR, NMID, self.mkOrderString() )

#      print qs

      if self.cursor.execute( qs ) == 0:
	 raise self.NoDataError

      self.data = self.cursor.fetchall()


   def query_names(self, nmlist):

      """Return a list of personID's for the names specified. If no names
      are specified, return an empty list of ID's (in that case, query will
      use all names available.)."""

      if not nmlist:
	 return []

      return _View.query_names( self, nmlist )


   def query_years(self, yrlist):

      """Return the list of years as is. No information will show up for
      years not in the database."""

      return yrlist


   def mkOrderString(self):

      """Return SQL ORDER string to use for query. It consists of a primary
      and secondary key, each with ascending or descending attribute.
      If date1 is not present in the specified sortorder, make date1 descending
      it's 3rd key.

      Keys can be: name, date1, date2, hours, category, remark

      String to generate: "ORDER BY key1 attr1, key2 attr2 [, date1 DESC]"
      """

      dict = {
	 'n':'name',
	 'c':'category',
	 '1':'date1',
	 '2':'date2',
	 'h':'hours',
	 'p':'part',
	 'r':'remark',
	 '+':'ASC',
	 '-':'DESC' }

      qs = "ORDER BY "

      # if no date1 in sort, make date1 descending it's 3rd key:

      if self.sort.find( '1' ) < 0:
	 self.sort += '1-'

#      print 'self.sort: ', self.sort

      for i in range(len(self.sort)):
	 if i > 0 and i % 2 == 0:
	    qs += ', '
	 qs += ' ' + dict[ self.sort[i] ]

      return qs


   def mkTable(self):

      """Create a table with verlof information collected."""

      self.mkTableHead()
      self.mkButtons()
      self.mkEntries()
      self.mkTableTail()


   def mkTableHead(self):
      """Start an HTML form/table."""
      print """<form name="menu" mothod="post" action="%s">
      <input type=hidden name="host" value="%s">
      <input type=hidden name="db"   value="%s">
      <input type=hidden name="sort" value="%s">
      <input type=hidden name="year" value="%s">
      <input type=hidden name="pname" value="%s">
      <input type=hidden name="kind" value="list">
      <table border=0 cellspacing=0>""" % ( self.uri, self.hostname, self.dbname, self.sort, self.year, self.name )


   def mkEntries(self):

      """Create all table rows with verlof information collected."""

      try:
	 self.query()
	 for row in self.data:
	    self.mkEntry( row )

      except self.NoDataError:
	 pass

#	  self.mkNoDataEntry( ('enig', '(naam)') )	# (any year, name)


   def mkEntry(self, row):

       """Create a single table row with verlof information collected."""

       nam, cat, frm, to, hrs, prt, rem = row
       prt = prt or ''
       rem = rem or ''

#	dtfmt = '%d %b %Y'
       dtfmt = '%d %m %Y'      # date format

       n = 2
       print '<tr>'
       print '<td valign=top>' + '&nbsp;'*n + nam + '</td>'
       print '<td valign=top>' + '&nbsp;'*n + self.catdesc(cat) + '</td>'
       print '<td valign=top class="right">' + frm.strftime(dtfmt) + '&nbsp;'*n + '</td>'
       print '<td valign=top>' + '&nbsp;'*n + to.strftime(dtfmt)  + '</td>'
       print '<td valign=top class="right">%g%s</td>' % ( hrs, '&nbsp;'*n )
       print '<td valign=top>' + '&nbsp;'*n + prt + '</td>'
       print '<td valign=top>' + rem.replace('\n', '<br>') + '</td>'
#	print '<td valign=top>' + '&nbsp;'*n + rem + '</td>'
       print '</tr>'


   def mkButtons(self):

      """Create a table row with title buttons. The buttons can be used
      to sort on each column,  alternating between ascending and descending."""

      print '<tr>'
      for info in [ ( 'namButton', 'naam'     , 25, string.ljust, "post( 'n' )" ),
		    ( 'catButton', 'categorie', 15, string.ljust, "post( 'c' )" ),
		    ( 'vanButton', 'van'      , 15, string.rjust, "post( '1' )" ),
		    (  'tmButton', 't/m'      , 15, string.ljust, "post( '2' )" ),
		    ( 'uurButton', 'uur'      ,  5, string.rjust, "post( 'h' )" ),
		    ( 'vnmButton', 'v/n'      ,  5, string.ljust, "post( 'p' )" ),
		    ( 'opmButton', 'opmerking', 30, string.ljust, "post( 'r' )" ) ]:
	 self.mkButton(info)
      print '</tr>'


   def mkButton(self, info):

      """Create a table cell with a single title button."""

      name, text, width, just, handler = info
      print '<td class="head"><input type="button" name="' + name + '" value="' + string.replace( just(text, width), ' ', '&nbsp;') + '" onclick="return ' + handler + '"></td>'


   def mkStyle(self):

      """Create CSS section's contents."""

      _View.mkStyle(self)
      print """<link rel="STYLESHEET" href="css/view-list.css" type="text/css">"""


   def mkScriptBody(self):

      """Create JavaScript section's contents."""

      _View.mkScriptBody(self)
      print """
      function mkSort(letter) {
	 // maintain a list of two sort entries, e.g.: 'n+1-', name ascending, date1 descending;
	 // pressing twice the same button toggles it's sort direction (+, or -)

	 sort = document.menu.sort.value
	 sign = sort.substr(1,1)

	 if ( letter == sort.substr(0,1) ) {
	    if ( sign == '+' ) sign = '-'
	    else	       sign = '+'

	    sort = letter + sign + sort.substr(2,2)
	 }
	 else sort = letter + '+' + sort.substr(0,2)

	 document.menu.sort.value = sort
//	   alert( 'sort.value: ' + document.menu.sort.value )
      }

      function post( letter ) {
//	   alert( 'post(' + letter + ')' )
	 mkSort( letter )
	 document.menu.submit()
	 return false
      }
      """


# ------------------------------------------------------------------------
# main line -
# ------------------------------------------------------------------------

def _error(msg):
   """Write message and traceback to to the Webserver error_log (sys.stderr).

   Optionally an e-mail is sent to the application administrator as
   configured through ./configure."""

   import traceback
   sys.stdout.write( msg )
   sys.stderr.write( 'view.py: error occurred, see traceback:\n' )
   traceback.print_exc( file=sys.stderr )

   _mailAdmin()

   sys.exit(0)


def _mailAdmin():
   """Send e-mail message with form and environment contents and traceback to administrator.

   config.appadmin holds the application administrator e-mail address,
   config.mailcmd specifies the mail program commandline.

   mail is only sent if both appadmin and mailcmd are non-empty."""

   if not config.appadmin or not config.mailcmd:
      return

   try:
      import traceback
      pipe=os.popen( config.mailcmd, 'w' )
      pipe.write   ( 'Error while executing view.py\n\n'      \
		     'Parameters:\n'  + `form`	     + '\n\n' \
		     'Environment:\n' + `os.environ` + '\n\n' )
      traceback.print_exc(file=pipe)
      pipe.close()
   except:
      pass


def _makeView():
   """Create the requested view."""

   #
   # first print HTTP header:
   #

   print "Content-Type: text/html"	# HTML is following
#   print "Cache-Control: no-cache"
#   print "Pragma: no-cache"
#   print "Expires: 0"
   print				# blank line, end of headers
   print				# non-empty response

   #
   # check if called from a real form entry:
   #

   if form.keys() == []:
      return

   #
   # For performance reasons, we do not route output via StringIO.
   # We accept partly produced output, followed by the error message.
   #

   try:
#      cgi.test()

      if os.environ.has_key('HTTP_USER_AGENT'): app  = os.environ['HTTP_USER_AGENT']
      if os.environ.has_key('REQUEST_URI'    ): uri  = os.environ['REQUEST_URI']

      kind   = form.getfirst('kind')
      dbhost = form.getfirst('dbhost')
      db     = form.getfirst('db'  )
      yr     = form.getfirst('year')
      nm     = form.getfirst('pname')
      so     = form.getfirst('sort')

      if not   kind:   kind = 'total'
      if not dbhost: dbhost = 'localhost'
      if not	 db:	 db = config.db
      if not	 so:	 so = 'n+1-'

      if   kind == 'list'     : view = ListView    ( dbhost, db, yr, nm, so, uri )
      elif kind == 'year'     : view = YearView    ( dbhost, db, yr, nm 	 )
      elif kind == 'total'    : view = TotalView   ( dbhost, db, yr, nm 	 )
      elif kind == 'totalcdf' : view = TotalViewCDF( dbhost, db, yr, nm, ';'	 )
      else		      : view = TotalViewCDF( dbhost, db, yr, nm, ';'	 )

      view.mkView()

   except:
      _error("""
<h1>view.py - probleem met verwerking</h1>
<p>De gevraagde informatie kan niet worden getoond, omdat er een fout optreedt als gevolg van uw verzoek.<br>
De fout is opgenomen in de log-files en er zal zo spoedig mogelijk aandacht aan worden gegeven.</p>
</body></html>""" )

   else:
      pass


_makeView()

#
# End of file
#

